import argparse
from zsub.common.version import __version__
from zsub.common.language import all_langs
from zsub.mirrors import MIRRORS

SORT_OPTIONS = ['t', 's', 'n']


def get_parser():
    examples = '''
Examples:
  zsub dl -t riverdale third season -l fa en -a
'''
    parser = argparse.ArgumentParser(
        prog="zsub",
        description='Downloads subtitles from subscene and its mirrors',
        formatter_class=argparse.RawTextHelpFormatter,
        add_help=True,
        epilog=examples)

    parser.add_argument(
        "--version",
        action="version",
        version=f'%(prog)s version {__version__}',
        help="show version and exit")
    commands = parser.add_subparsers(
        dest="command",
        help='available commands')
    download = commands.add_parser(
        'dl',
        help='download')
    unzip = commands.add_parser(
        'unzip',
        help='Extract downloaded title')
    info = commands.add_parser(
        'info',
        help='Show Useful Information')
    batch = commands.add_parser(
        'batch',
        help='Batch mode download')
    config = commands.add_parser(
        'config',
        help='Set default configuration to config.ini')
    clean = commands.add_parser(
        'clean',
        help='delete downloaded subs for a title')
    commands.add_parser(
        'login',
        help='login and get required cookies values.')
    commands.add_parser(
        'update',
        help="update zsub")

    # download
    download_args = download.add_argument_group(title="Arguments")
    download_args.add_argument(
        '-t',
        dest="title",
        nargs='+',
        metavar='TITLE',
        help='title to search')
    download_args.add_argument(
        '-l',
        dest='languages',
        nargs='+',
        choices=all_langs(),
        metavar=('LNG1', 'LNG2',),
        help='Language(s) to download')
    download_args.add_argument(
        '-s',
        dest='site',
        choices=MIRRORS,
        metavar="Site",
        help='Site to download from.')
    download_args.add_argument(
        '-d',
        dest='destination',
        metavar="DESTINATION",
        help='Destination folder to extract downloaded files to.')
    download_group = download_args.add_mutually_exclusive_group()
    download_group.add_argument(
        '-g',
        dest='group',
        action='store_true',
        help='group subtitles at destination')
    download_group.add_argument(
        '-G',
        dest='no_group',
        action='store_false',
        help='do not group subtitles at destination')
    download_ask = download_args.add_mutually_exclusive_group()
    download_ask.add_argument(
        '-a',
        '--auto-select',
        dest='auto_select',
        action='store_true',
        help='Select first title in search results automatically.')
    download_ask.add_argument(
        '-A',
        '--ask',
        dest='auto_select',
        action='store_false',
        help='Ask user to select best match in search results.')
    download_open = download_args.add_mutually_exclusive_group()
    download_open.add_argument(
        '-o',
        dest='open_after',
        action='store_true',
        help='Open destination folder after.')
    download_open.add_argument(
        '-O',
        dest='open_after',
        action='store_false',
        help='Do not open destination folder after.')
    download_args.add_argument(
        '-S',
        '--simulation',
        dest='simulation',
        action='store_true',
        help='Do not download, just create empty zip files.')

    # unzip
    unzip_args = unzip.add_argument_group(title="Arguments")
    unzip_args.add_argument(
        "-t",
        dest='title',
        nargs='+',
        help="extract downloaded files related to search keyword.")
    unzip_args.add_argument(
        '-l',
        dest='languages',
        nargs='+',
        choices=all_langs(),
        metavar=('LNG1', 'LNG2',),
        help='Language(s) to extract.')
    unzip_args.add_argument(
        '-d',
        dest='destination',
        metavar="DESTINATION",
        help='Destination folder to extract files to.')
    unzip_group = unzip_args.add_mutually_exclusive_group()
    unzip_group.add_argument(
        '-g',
        dest='group',
        action='store_true',
        help='group subtitles at destination')
    unzip_group.add_argument(
        '-G',
        dest='no_group',
        action='store_false',
        help='do not group subtitles at destination')
    unzip_ask = unzip_args.add_mutually_exclusive_group()
    unzip_ask.add_argument(
        '-a',
        '--auto-select',
        dest='auto_select',
        action='store_true',
        help='Select first title in search results automatically.')
    unzip_ask.add_argument(
        '-A',
        '--ask',
        dest='auto_select',
        action='store_false',
        help='Ask user to select best match in search results.')
    unzip_open = unzip_args.add_mutually_exclusive_group()
    unzip_open.add_argument(
        '-o',
        dest='open_after',
        action='store_true',
        help='Open destination folder after.')
    unzip_open.add_argument(
        '-O',
        dest='open_after',
        action='store_false',
        help='Do not pen destination folder after.')

    # info
    info_args = info.add_argument_group(title="Arguments")
    info_args.add_argument(
        '-v',
        dest='verbosity',
        action='count',
        help='increase verbosity. use -v, -vv, -vvv ...')
    info_sort = info_args.add_mutually_exclusive_group()
    info_sort.add_argument(
        "-t",
        action='store_true',
        dest='title_sort',
        help="sort output based on titles. (only with -v)")
    info_sort.add_argument(
        "-n",
        action='store_true',
        dest='number_sort',
        help="sort output based on number of files of a title. (only with -v)")
    info_sort.add_argument(
        "-s",
        action='store_true',
        dest='size_sort',
        help="sort output based on size of files of a title. (only with -v)")

    # batch
    batch_args = batch.add_argument_group(title="Arguments")
    batch_commands = batch_args.add_mutually_exclusive_group()
    batch_commands.add_argument(
        "--print",
        dest='print',
        action='store_true',
        help="print content of batch list")
    batch_commands.add_argument(
        "--reset",
        dest='reset',
        action='store_true',
        help="overwrites batch.ini with a empty template.")
    batch_commands.add_argument(
        "--add",
        dest='add',
        action='store_true',
        help="cli for add titles to download.")
    batch_args.add_argument(
        '-G',
        dest='no_group',
        action='store_true',
        help='do not group subtitles at destination')
    batch_args.add_argument(
        '-s',
        dest='site',
        choices=MIRRORS,
        metavar="Site",
        help='Site to download/search from.')
    batch_args.add_argument(
        '-d',
        dest='destination',
        metavar="DESTINATION",
        help='Destination folder to extract files to.')
        
    # config
    config_args = config.add_argument_group(title="Arguments")
    config_args.add_argument(
        "-r",
        "--reset",
        dest='reset',
        action='store_true',
        help="reset config.ini to default values")

    # clean
    clean_args = clean.add_argument_group(title="Arguments")
    clean_args.add_argument(
        "-t",
        dest='title',
        nargs='+',
        help="delete downloaded files related to search keyword.")
    clean_args.add_argument(
        "-0",
        "--zero",
        dest='zero',
        action='store_true',
        help="do not delete, just empty the files.")
    clean_args.add_argument(
        '-l',
        dest='languages',
        nargs='+',
        choices=all_langs(),
        metavar=('LNG1', 'LNG2',),
        help='Language(s) to delete.')

    return parser
