import os
from configparser import ConfigParser
from pathlib import Path
from zsub.common.utils import OS, get_captcha, get_last_check
from zsub.common.language import all_langs, Language
from zsub.mirrors import MIRRORS, get_mirror


SITE = 'subscene'
LANGUAGES = 'en'
OPEN_AFTER = True
AUTO_SELECT = False
if OS == "Windows":
    CACHE  = Path(os.environ['PROGRAMDATA']).joinpath('.', 'zsub', 'cache')
    CONFIG = Path(os.environ['PROGRAMDATA']).joinpath('.', 'zsub', 'config')
else:
    CACHE  = Path().home().joinpath('.', '.zsub', 'cache')
    CONFIG = Path().home().joinpath('.', '.zsub', 'config')
UNZIP  = Path().home().joinpath('.', 'Downloads', 'zsub')

class Configs:
    def __init__(self):
        self.site = get_mirror(SITE)
        self.auto_select = AUTO_SELECT
        self.open_after = OPEN_AFTER
        self.languages = Language(*LANGUAGES.split())
        self.cache = CACHE
        self.config = CONFIG
        self.unzip = UNZIP
        self.captcha = get_captcha(CONFIG)
        self.last_check = get_last_check(CONFIG)

def get_configs(path, display=False):
    configs = ConfigParser()
    if path.exists():
        if is_valid(path):
            configs.read(path)
            # to avoid crash if user manually deletes lines in config.ini
            user = configs['User']
            user['open_after'] = str(user.getboolean('open_after', OPEN_AFTER))
            user['auto_select'] = str(user.getboolean('auto_select', AUTO_SELECT))
            user['site'] = str(user.get('site', SITE))
            user['languages'] = str(user.get('languages', LANGUAGES))
            user['unzip_folder'] = str(user.get('unzip_folder', UNZIP.absolute()))
        else:
            print('\n[config] file content is changed and not valid. Please change values only by calling "zsub config".')
            print_configs(path)
            print('[config] creating a fresh valid one. old one is renamed to "xconfig.ini"')
            path.replace(path.parent.joinpath('.', 'xconfig.ini'))
            configs = fresh_configs(path)
    else:
        print('\n[config] file does not exist. creating with default values.')
        configs = fresh_configs(path)
    if display:
        print_configs(path)
    return configs

def print_configs(path):
    with open(path, 'r') as config_file:
        text = config_file.read()
        lines = [line for line in text.splitlines() if line]
        print('')
        print('-------config.ini----------')
        print('\n'.join(lines))
        print('----------------------------')

def fresh_configs(path, display=False):
    configs = ConfigParser()
    configs['User'] = {
        'open_after' : OPEN_AFTER,
        'auto_select' : AUTO_SELECT,
        'site' : SITE,
        'languages' : LANGUAGES,
        'unzip_folder' : UNZIP
    }
    path.parent.mkdir(parents=True, exist_ok=True)
    with open(path, 'w+') as configfile:
        configs.write(configfile)
    if display:
        print_configs(path)
    return configs

def is_valid(path):
    try:
        config = ConfigParser()
        config.read(path)
        user = config['User']
        user.getboolean('open_after')
        user.getboolean('auto_select')
        user.get('site')
        user.get('languages')
        user.get('unzip_folder')
        return True
    except:
        return False

def review_configs(path):
    configs = get_configs(path, display=True)
    user = configs['User']
    open_after = user.get('open_after')
    auto_select = user.get('auto_select')
    site = user.get('site')
    languages = user.get('languages')
    unzip_folder = user.get('unzip_folder')
    user['open_after'] = get_boolean_choice('open after?', open_after)
    user['auto_select'] = get_boolean_choice('auto select?', auto_select)
    user['languages'] = get_language_choice(languages)
    user['site'] = get_site_choice(site)
    user['unzip_folder'] = get_path_choice(unzip_folder)
    with open(path, 'w+') as config_file:
        configs.write(config_file)

    print_configs(path)

def get_boolean_choice(text, current): 
    print('\n' + text)
    print('valid answers: {true, yes, on, 1, false, no, off, 0}')
    answer = input(f'[{current}]: ') or current
    while str(answer).lower() not in {'true', 'yes', 'on', '1', 'false', 'no', 'off', '0'}:
        print('not an option. try again.')
        answer = input(text+f' [{current}]: ') or current
    return str(answer)

def get_language_choice(current):
    langs = ''
    print('\n' + 'language(s) in abbr, space separated.')
    print('valid answers: {' + ', '.join(all_langs())+ '}')
    answer = (input(f'[{current}]: ') or current).split(' ')
    for ans in answer:
        if ans in all_langs():
            langs += str(ans) + ' '
        else:
            print(f'"{ans}" is not a valid language. Ignored.')
    return langs or str(current)

def get_site_choice(current):
    print('\n' + 'default site.')
    print('valid answers: {' + ', '.join(MIRRORS) + '}')
    answer = input(f'[{current}]: ') or current
    while str(answer) not in MIRRORS:
        print('not an option. try again.')
        answer = input(f'[{current}]: ') or current
    return str(answer)

def get_path_choice(current):
    print('\n' + 'default folder path to extract subtitles to.')
    while True:
        path = Path(input(f'[{current}]: ') or current)
        if path.exists():
            return str(path.absolute())
        else:
            confirm = input(f'"{path.absolute()}" does not exist. create? [Y/n]: ') or 'Y'
            if confirm.lower() == 'y':
                path.mkdir(exist_ok=True, parents=True)
                return str(path.absolute())
            else:
                print('try again.')


