import platform
import subprocess
import shutil
import json
from zipfile import ZipFile
from hashlib import md5
from datetime import datetime


OUTDATE = '1546300800' # 2019-01-01
OS = platform.system()


def open_folder(path, open_after):
    if open_after:
        if OS == "Windows":
            subprocess.Popen(["explorer", str(path)])
        elif OS == "Darwin":
            subprocess.Popen(["open", path])
        else:
            subprocess.Popen(["xdg-open", path])
        return OS
    else:
        return None

def get_size(cache):
    if OS == "Windows":
        size = subprocess.check_output(
            ['powershell',
             '-noprofile',
             '-command',
             f"ls {cache} -r | measure -s Length | select -ExpandProperty Sum"]
        )
        if not size:
            size = '0'
        return human_readable(int(size))
    else:
        size = subprocess.check_output(
            ['du', '-s', '-B 1', cache]).split()[0].decode('utf-8')
        return human_readable(int(size))


def human_readable(size):
    for unit in ['', 'K', 'M', 'G', 'T']:
        if size < 1024.0:
            return f"{size:03.2f} {unit}B"
        size = size / 1024.0


def byte_equivalent(size):
    units = {"B": 1, "KB": 1024, "MB": 1024**2, "GB": 1024**3, "TB": 1024**4}
    number, unit = size.split()
    return int(float(number)*units[unit])


def get_words(text):
    words = str(text).lower().replace('+', ' ').replace('-', ' ')
    return set(words.split())


def get_score(text1, text2):
    words1 = get_words(text1)
    words2 = get_words(text2)
    score = 0
    for w1 in words1:
        for w2 in words2:
            if w1 == w2:
                score += 1
                break
    return score


def get_title(path):
    return " ".join(path.stem.split('-')).title()


def lpad(x, mx):
    return str(x).rjust(len(str(mx)), " ")


def get_close_matches(path, title):
    max_score = 1
    titles = list()
    for child in path.iterdir():
        score = get_score(title, child.name)
        item = {'path': child, 'title': get_title(child)}
        if score > max_score:
            max_score = score
            titles = [item, ]
        elif score == max_score:
            titles.append(item)
    return titles


def empty(path):
    for child in path.iterdir():
        with ZipFile(child, 'w'):
            pass
        print(f'emptied: {child.absolute()}')


def delete(path):
    shutil.rmtree(path)
    print(f'deleted: {path.absolute()}')


def delete_if_empty(path):
    if isempty(path):
        print(f'deleted: [empty folder] {path.absolute()}')
        path.rmdir()


def clean_root(root):
    for title in root.iterdir():
        for langs in title.iterdir():
            delete_if_empty(langs)
        delete_if_empty(title)
    print('')


def isempty(path):
    for _ in path.iterdir():
        return False
    return True


def count_children(path):
    ch = 0
    for _ in path.iterdir():
        ch += 1
    return ch


def count_grandchildren(path):
    ch = 0
    for child in path.iterdir():
        ch += count_children(child)
    return ch


def count_total(path):
    ch = 0
    for child in path.iterdir():
        ch += count_grandchildren(child)
    return ch


def get_sorted_items(path, sort_key):
    items = list()
    for child in path.iterdir():
        items.append({
            't': get_title(child),
            's': get_size(child),
            'n': count_grandchildren(child)
        })

    def key(x):
        if sort_key == 's':
            return byte_equivalent(x[sort_key])
        else:
            return x[sort_key]

    reverse = False if sort_key == 't' else True
    return sorted(items, key=key, reverse=reverse)

def remove_same(paths):
    to_remove = []
    for path in paths:
        files = [{'file': item, 'md5': md5(item.read_bytes()).digest()} for item in path.iterdir() if item.is_file() ]
        i = 0
        for file1 in files:
            for file2 in files[i+1:]:
                if file1['md5'] == file2['md5']:
                    to_remove.append(file1['file'])
                    break
            i +=1
    
    if to_remove:
        print('\n[Remove Duplicates]')
        for file in to_remove:
            print(f"sub: {file}")
            file.unlink()
    print()
    
def get_captcha(path):
    path.mkdir(parents=True, exist_ok=True)
    path = path.joinpath('.', 'captcha.txt')
    if not path.exists():
        with open(path, 'w'):
            pass
    with open(path, 'r') as file:
        return file.read()

def set_captcha(path, value):
    path.mkdir(parents=True, exist_ok=True)
    captcha = path.joinpath('.', 'captcha.txt')
    with open(captcha, 'w') as file:
        file.write(value)

def need_update(package):
    if OS == 'Windows':
        pip = 'pip'
    else:
        pip = 'pip3'
    outdated = json.loads(subprocess.check_output([pip, 'list', '-o', '--format=json']))
    for pkg in outdated:
        if pkg['name'] == package:
            return pkg
    return False

def print_update_info(package):
    info = []
    info.append(f'There is a new version: {package["version"]} --> {package["latest_version"]}')
    info.append('Upgrade with:')
    info.append('')
    if OS == 'Windows':
        info.append(f'pip install -U {package["name"]}')
    else:
        info.append(f'pip3 install -U --user {package["name"]}')
    max_length = max([len(s) for s in info])
    print('*'*(max_length+8))
    for line in info:
        print(f'*** {line}{" "*(max_length-len(line))} ***')
    print('*'*(max_length+8))

def get_last_check(path):
    path.mkdir(parents=True, exist_ok=True)
    last = path.joinpath('.', 'last-check')
    if not last.exists():
        set_last_check(path, OUTDATE)
    with open(last, 'r') as file:
        try:
            last_check = datetime.fromtimestamp(int(file.readline()))
        except:
            set_last_check(path, OUTDATE)
            last_check = datetime.fromtimestamp(1)
    return last_check

def set_last_check(path, value=None):
    path.mkdir(parents=True, exist_ok=True)
    last = path.joinpath('.', 'last-check')
    with open(last, 'w') as file:
        if value:
            file.write(value)
        else:
            file.write(str(int(datetime.now().timestamp())))

def is_time_to_check(last_check, period):
    if (datetime.now() - last_check).days >= period:
        return True
    else:
        return False 
